@@ -11,8 +11,8 @@ use std::{
     collections::{BTreeMap, BTreeSet, HashMap, HashSet},
     convert::TryInto,
     fmt::{self, Debug, Formatter},
-    fs, io,
-    path::PathBuf,
+    fs,
+    path::{Path, PathBuf},
     sync::Arc,
     time::Duration,
 };
@@ -104,8 +104,8 @@ pub struct EraSupervisor<I> {
     next_executed_height: u64,
     #[data_size(skip)]
     metrics: ConsensusMetrics,
-    /// The path to the folder where unit hash files will be stored.
-    unit_hashes_folder: PathBuf,
+    /// The path to the folder where unit files will be stored.
+    unit_files_folder: PathBuf,
     /// The next upgrade activation point. When the era immediately before the activation point is
     /// deactivated, the era supervisor indicates that the node should stop running to allow an
     /// upgrade.
@@ -131,6 +131,7 @@ where
     #[allow(clippy::too_many_arguments)]
     pub(crate) fn new<REv: ReactorEventT<I>>(
         current_era: EraId,
+        storage_dir: &Path,
         config: WithDir<Config>,
         effect_builder: EffectBuilder<REv>,
         protocol_config: ProtocolConfig,
@@ -146,7 +147,7 @@ where
                 current_era, protocol_config.last_activation_point
             );
         }
-        let unit_hashes_folder = config.with_dir(config.value().highway.unit_hashes_folder.clone());
+        let unit_files_folder = storage_dir.join("unit_files");
         let (root, config) = config.into_parts();
         let (secret_signing_key, public_signing_key) = config.load_keys(root)?;
         info!(our_id = %public_signing_key, "EraSupervisor pubkey",);
@@ -167,7 +168,7 @@ where
             new_consensus,
             next_block_height: next_height,
             metrics,
-            unit_hashes_folder,
+            unit_files_folder,
             next_upgrade_activation_point,
             stop_for_upgrade: false,
             next_executed_height: next_height,
@@ -354,7 +355,7 @@ where
                 our_id.clone(),
                 secret,
                 now,
-                Some(self.unit_hash_file(&instance_id)),
+                Some(self.unit_file(&instance_id)),
             ))
         }
 
@@ -399,21 +400,58 @@ where
         // units that refer to evidence from any era that was bonded when it was the current one.
         let oldest_evidence_era_id = oldest_bonded_era(&self.protocol_config, oldest_bonded_era_id);
         if let Some(obsolete_era_id) = oldest_evidence_era_id.checked_sub(1) {
-            if let Some(era) = self.active_eras.remove(&obsolete_era_id) {
+            if let Some(_era) = self.active_eras.remove(&obsolete_era_id) {
                 trace!(era = obsolete_era_id.value(), "removing obsolete era");
-                match fs::remove_file(self.unit_hash_file(era.consensus.instance_id())) {
-                    Ok(_) => {}
-                    Err(err) => match err.kind() {
-                        io::ErrorKind::NotFound => {}
-                        err => warn!(?err, "could not delete unit hash file"),
-                    },
-                }
             }
         }
 
         outcomes
     }
 
+    fn prune_unit_files(&self) {
+        let valid_unit_files: HashSet<_> = self
+            .active_eras
+            .iter()
+            .map(|(_, era)| self.unit_file(era.consensus.instance_id()))
+            .collect();
+
+        let dir_iterator = match fs::read_dir(&self.unit_files_folder) {
+            Ok(iter) => iter,
+            Err(err) => {
+                warn!(?err, path=?self.unit_files_folder, "could not read the unit files folder");
+                // if we couldn't clean up the unit files, we just return
+                return;
+            }
+        };
+
+        for entry in dir_iterator {
+            let entry = match entry {
+                Ok(entry) => entry,
+                Err(err) => {
+                    warn!(
+                        ?err,
+                        path=?self.unit_files_folder,
+                        "error while reading the unit files folder",
+                    );
+                    continue;
+                }
+            };
+            let path = entry.path();
+            if path.is_dir() {
+                // unit files should be stored directly in the folder, not in subdirectories
+                continue;
+            }
+            if valid_unit_files.contains(&path) {
+                // don't remove files corresponding to active eras
+                continue;
+            }
+            debug!(?path, "removing unit file");
+            if let Err(err) = fs::remove_file(&path) {
+                warn!(?err, ?path, "could not delete unit file");
+            }
+        }
+    }
+
     /// Returns `true` if the specified era is active and bonded.
     fn is_bonded(&self, era_id: EraId) -> bool {
         era_id.saturating_add(self.bonded_eras().into()) >= self.current_era
@@ -537,6 +575,9 @@ where
             );
             result_map.insert(era_id, results);
         }
+
+        self.prune_unit_files();
+
         let active_era_outcomes = self.active_eras[&self.current_era]
             .consensus
             .handle_is_current(now);
@@ -565,10 +606,10 @@ where
         self.bonded_eras().min(self.protocol_config.auction_delay)
     }
 
-    /// Returns the path to the era's unit hash file.
-    fn unit_hash_file(&self, instance_id: &Digest) -> PathBuf {
-        self.unit_hashes_folder.join(format!(
-            "unit_hash_{:?}_{}.dat",
+    /// Returns the path to the era's unit file.
+    fn unit_file(&self, instance_id: &Digest) -> PathBuf {
+        self.unit_files_folder.join(format!(
+            "unit_{:?}_{}.dat",
             instance_id,
             self.public_signing_key.to_hex()
         ))
@@ -700,7 +741,10 @@ where
     /// Applies `f` to the consensus protocol of the specified era.
     fn delegate_to_era<F>(&mut self, era_id: EraId, f: F) -> Effects<Event<I>>
     where
-        F: FnOnce(&mut dyn ConsensusProtocol<I, ClContext>) -> Vec<ProtocolOutcome<I, ClContext>>,
+        F: FnOnce(
+            &mut dyn ConsensusProtocol<I, ClContext>,
+            &mut NodeRng,
+        ) -> Vec<ProtocolOutcome<I, ClContext>>,
     {
         match self.era_supervisor.active_eras.get_mut(&era_id) {
             None => {
@@ -712,7 +756,7 @@ where
                 Effects::new()
             }
             Some(era) => {
-                let outcomes = f(&mut *era.consensus);
+                let outcomes = f(&mut *era.consensus, self.rng);
                 self.handle_consensus_outcomes(era_id, outcomes)
             }
         }
@@ -724,7 +768,7 @@ where
         timestamp: Timestamp,
         timer_id: TimerId,
     ) -> Effects<Event<I>> {
-        self.delegate_to_era(era_id, move |consensus| {
+        self.delegate_to_era(era_id, move |consensus, _| {
             consensus.handle_timer(timestamp, timer_id)
         })
     }
@@ -734,7 +778,7 @@ where
         era_id: EraId,
         action_id: ActionId,
     ) -> Effects<Event<I>> {
-        self.delegate_to_era(era_id, move |consensus| {
+        self.delegate_to_era(era_id, move |consensus, _| {
             consensus.handle_action(action_id, Timestamp::now())
         })
     }
@@ -745,8 +789,8 @@ where
                 // If the era is already unbonded, only accept new evidence, because still-bonded
                 // eras could depend on that.
                 trace!(era = era_id.value(), "received a consensus message");
-                self.delegate_to_era(era_id, move |consensus| {
-                    consensus.handle_message(sender, payload, Timestamp::now())
+                self.delegate_to_era(era_id, move |consensus, rng| {
+                    consensus.handle_message(rng, sender, payload, Timestamp::now())
                 })
             }
             ConsensusMessage::EvidenceRequest { era_id, pub_key } => {
@@ -757,7 +801,7 @@ where
                 self.era_supervisor
                     .iter_past(era_id, self.era_supervisor.bonded_eras())
                     .flat_map(|e_id| {
-                        self.delegate_to_era(e_id, |consensus| {
+                        self.delegate_to_era(e_id, |consensus, _| {
                             consensus.request_evidence(sender.clone(), &pub_key)
                         })
                     })
@@ -780,7 +824,7 @@ where
             return Effects::new();
         }
         let proposed_block = ProposedBlock::new(block_payload, block_context);
-        self.delegate_to_era(era_id, move |consensus| {
+        self.delegate_to_era(era_id, move |consensus, _| {
             consensus.propose(proposed_block, Timestamp::now())
         })
     }
@@ -944,6 +988,7 @@ where
             switch_block_header.timestamp(),
             switch_block_header.height() + 1,
         );
+        self.era_supervisor.prune_unit_files();
         outcomes.extend(
             self.era_supervisor.active_eras[&era_id]
                 .consensus
@@ -978,7 +1023,7 @@ where
             .get_mut(&era_id)
             .map_or(false, |era| era.resolve_validity(&proposed_block, valid))
         {
-            effects.extend(self.delegate_to_era(era_id, |consensus| {
+            effects.extend(self.delegate_to_era(era_id, |consensus, _| {
                 consensus.resolve_validity(proposed_block, valid, Timestamp::now())
             }));
         }
@@ -1214,7 +1259,7 @@ where
                             continue;
                         };
                     for proposed_block in proposed_blocks {
-                        effects.extend(self.delegate_to_era(e_id, |consensus| {
+                        effects.extend(self.delegate_to_era(e_id, |consensus, _| {
                             consensus.resolve_validity(proposed_block, true, Timestamp::now())
                         }));
                     }
@@ -1225,7 +1270,7 @@ where
                 .era_supervisor
                 .iter_past_other(era_id, self.era_supervisor.bonded_eras())
                 .flat_map(|e_id| {
-                    self.delegate_to_era(e_id, |consensus| {
+                    self.delegate_to_era(e_id, |consensus, _| {
                         consensus.request_evidence(sender.clone(), &pub_key)
                     })
                 })
