@@ -43,7 +43,7 @@ use casper_types::ProtocolVersion;
 use super::Component;
 use crate::{
     effect::{EffectBuilder, Effects},
-    reactor::participating::Event as ParticipatingReactorEvent,
+    reactor::participating::ParticipatingEvent as ParticipatingReactorEvent,
     types::JsonBlock,
     utils::{self, ListeningError},
     NodeRng,
@@ -194,6 +194,11 @@ where
                 block_hash: Box::new(block_hash),
                 execution_result,
             }),
+            Event::DeploysExpired(deploy_hashes) => deploy_hashes
+                .into_iter()
+                .map(|deploy_hash| self.broadcast(SseData::DeployExpired { deploy_hash }))
+                .flatten()
+                .collect(),
             Event::Fault {
                 era_id,
                 public_key,
@@ -204,9 +209,12 @@ where
                 timestamp,
             }),
             Event::FinalitySignature(fs) => self.broadcast(SseData::FinalitySignature(fs)),
-            Event::Step { era_id, effect } => self.broadcast(SseData::Step {
+            Event::Step {
                 era_id,
-                execution_effect: effect,
+                execution_effect,
+            } => self.broadcast(SseData::Step {
+                era_id,
+                execution_effect,
             }),
         }
     }
