@@ -5,7 +5,7 @@
 
 mod era_summary;
 
-use std::str;
+use std::{num::ParseIntError, str};
 
 use futures::{future::BoxFuture, FutureExt};
 use http::Response;
@@ -76,6 +76,41 @@ pub enum BlockIdentifier {
     Height(u64),
 }
 
+impl str::FromStr for BlockIdentifier {
+    type Err = ParseBlockIdentifierError;
+
+    fn from_str(maybe_block_identifier: &str) -> Result<Self, Self::Err> {
+        if maybe_block_identifier.is_empty() {
+            return Err(ParseBlockIdentifierError::EmptyString);
+        }
+
+        if maybe_block_identifier.len() == (Digest::LENGTH * 2) {
+            let hash = Digest::from_hex(maybe_block_identifier)
+                .map_err(ParseBlockIdentifierError::FromHexError)?;
+            Ok(BlockIdentifier::Hash(BlockHash::new(hash)))
+        } else {
+            let height = maybe_block_identifier
+                .parse()
+                .map_err(ParseBlockIdentifierError::ParseIntError)?;
+            Ok(BlockIdentifier::Height(height))
+        }
+    }
+}
+
+/// Represents errors that can arise when parsing a [`BlockIdentifier`].
+#[derive(thiserror::Error, Debug)]
+pub enum ParseBlockIdentifierError {
+    /// String was empty.
+    #[error("Empty string is not a valid block identifier.")]
+    EmptyString,
+    /// Couldn't parse a height value.
+    #[error("Unable to parse height from string. {0}")]
+    ParseIntError(ParseIntError),
+    /// Couldn't parse a blake2bhash.
+    #[error("Unable to parse digest from string. {0}")]
+    FromHexError(casper_hashing::Error),
+}
+
 /// Params for "chain_get_block" RPC request.
 #[derive(Serialize, Deserialize, Debug, JsonSchema)]
 #[serde(deny_unknown_fields)]
@@ -423,7 +458,7 @@ impl RpcWithOptionalParamsExt for GetEraInfoBySwitchBlock {
                     era_id,
                     stored_value,
                     state_root_hash,
-                    merkle_proof: hex::encode(proof_bytes),
+                    merkle_proof: base16::encode_lower(&proof_bytes),
                 }),
             };
 
