@@ -4,11 +4,18 @@ use std::{
     sync::Arc,
 };
 
-use casper_types::{ProtocolVersion, PublicKey, SecretKey, Signature};
+use casper_types::{AsymmetricType, ProtocolVersion, PublicKey, SecretKey, Signature};
 use datasize::DataSize;
-use serde::{de::DeserializeOwned, Deserialize, Serialize};
+use serde::{
+    de::{DeserializeOwned, Error as SerdeError},
+    Deserialize, Deserializer, Serialize, Serializer,
+};
 
 use crate::crypto;
+#[cfg(test)]
+use crate::crypto::AsymmetricKeyExt;
+#[cfg(test)]
+use crate::testing::TestRng;
 
 use super::counting_format::ConnectionId;
 
@@ -19,6 +26,7 @@ fn default_protocol_version() -> ProtocolVersion {
 }
 
 #[derive(Clone, Debug, Deserialize, Serialize)]
+#[allow(clippy::large_enum_variant)]
 pub(crate) enum Message<P> {
     Handshake {
         /// Network we are connected to.
@@ -77,7 +85,11 @@ impl ConsensusKeyPair {
 }
 
 /// Certificate used to indicate that the peer is a validator using the specified public key.
-#[derive(Clone, Debug, Deserialize, Serialize)]
+///
+/// Note that this type has custom `Serialize` and `Deserialize` implementations to allow the
+/// `public_key` and `signature` fields to be encoded to all-lowercase hex, hence circumventing the
+/// checksummed-hex encoding used by `PublicKey` and `Signature` normally.
+#[derive(Clone, Debug, Eq, PartialEq)]
 pub(crate) struct ConsensusCertificate {
     public_key: PublicKey,
     signature: Signature,
@@ -98,6 +110,17 @@ impl ConsensusCertificate {
         crypto::verify(connection_id.as_bytes(), &self.signature, &self.public_key)?;
         Ok(self.public_key)
     }
+
+    /// Creates a random `ConnectionId`.
+    #[cfg(test)]
+    fn random(rng: &mut TestRng) -> Self {
+        let secret_key = SecretKey::random(rng);
+        let public_key = PublicKey::from(&secret_key);
+        ConsensusCertificate::create(
+            ConnectionId::random(rng),
+            &ConsensusKeyPair::new(Arc::new(secret_key), public_key),
+        )
+    }
 }
 
 impl Display for ConsensusCertificate {
@@ -106,6 +129,87 @@ impl Display for ConsensusCertificate {
     }
 }
 
+/// This type and the `NonHumanReadableCertificate` are helper structs only used in the `Serialize`
+/// and `Deserialize` implementations of `ConsensusCertificate` to allow handshaking between nodes
+/// running the casper-node v1.4.2 software and later versions.
+///
+/// Checksummed-hex encoding was introduced in 1.4.2 and is applied to `PublicKey` and `Signature`
+/// types, affecting the encoding of `ConsensusCertificate` since handshaking uses a human-readable
+/// type of encoder/decoder.
+///
+/// The version immediately after 1.4.2 and subsequent versions use a slightly different style of
+/// checksummed-hex encoding which is incompatible with the 1.4.2 style.  To effectively disable
+/// checksummed-hex encoding, we simply need to use an all-lowercase form of hex encoding for the
+/// `PublicKey` and `Signature` types.
+///
+/// The `HumanReadableCertificate` enables that by explicitly being constructed from all-lowercase
+/// hex encoded types, while the `NonHumanReadableCertificate` is a simple mirror of
+/// `ConsensusCertificate` to allow us to derive `Serialize` and `Deserialize`, avoiding complex
+/// hand-written implementations for the non-human-readable case.
+#[derive(Serialize, Deserialize)]
+struct HumanReadableCertificate {
+    public_key: String,
+    signature: String,
+}
+
+#[derive(Serialize, Deserialize)]
+struct NonHumanReadableCertificate {
+    public_key: PublicKey,
+    signature: Signature,
+}
+
+impl Serialize for ConsensusCertificate {
+    fn serialize<S: Serializer>(&self, serializer: S) -> Result<S::Ok, S::Error> {
+        if serializer.is_human_readable() {
+            let human_readable_certificate = HumanReadableCertificate {
+                public_key: self.public_key.to_hex().to_lowercase(),
+                signature: self.signature.to_hex().to_lowercase(),
+            };
+
+            return human_readable_certificate.serialize(serializer);
+        }
+
+        let non_human_readable_certificate = NonHumanReadableCertificate {
+            public_key: self.public_key.clone(),
+            signature: self.signature,
+        };
+        non_human_readable_certificate.serialize(serializer)
+    }
+}
+
+impl<'de> Deserialize<'de> for ConsensusCertificate {
+    fn deserialize<D: Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
+        if deserializer.is_human_readable() {
+            let human_readable_certificate = HumanReadableCertificate::deserialize(deserializer)?;
+            let public_key = PublicKey::from_hex(
+                human_readable_certificate
+                    .public_key
+                    .to_lowercase()
+                    .as_bytes(),
+            )
+            .map_err(D::Error::custom)?;
+            let signature = Signature::from_hex(
+                human_readable_certificate
+                    .signature
+                    .to_lowercase()
+                    .as_bytes(),
+            )
+            .map_err(D::Error::custom)?;
+            return Ok(ConsensusCertificate {
+                public_key,
+                signature,
+            });
+        }
+
+        let non_human_readable_certificate =
+            NonHumanReadableCertificate::deserialize(deserializer)?;
+        Ok(ConsensusCertificate {
+            public_key: non_human_readable_certificate.public_key,
+            signature: non_human_readable_certificate.signature,
+        })
+    }
+}
+
 impl<P: Display> Display for Message<P> {
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
         match self {
@@ -205,7 +309,7 @@ mod tests {
 
     use crate::protocol;
 
-    use super::Message;
+    use super::*;
 
     /// Version 1.0.0 network level message.
     ///
@@ -230,6 +334,24 @@ mod tests {
         101, 115, 116, 177, 49, 50, 46, 51, 52, 46, 53, 54, 46, 55, 56, 58, 49, 50, 51, 52, 54,
     ];
 
+    /// A "conserved" version 1.4.2 handshake.
+    ///
+    /// NEVER CHANGE THIS CONSTANT TO MAKE TESTS PASS, AS IT IS BASED ON TESTNET DATA.
+    const V1_4_2_HANDSHAKE: &[u8] = &[
+        129, 0, 148, 177, 101, 120, 97, 109, 112, 108, 101, 45, 104, 97, 110, 100, 115, 104, 97,
+        107, 101, 177, 49, 50, 46, 51, 52, 46, 53, 54, 46, 55, 56, 58, 49, 50, 51, 52, 54, 165, 49,
+        46, 52, 46, 50, 146, 217, 68, 48, 50, 48, 50, 56, 51, 99, 48, 68, 54, 56, 55, 57, 51, 51,
+        69, 98, 50, 48, 97, 53, 52, 49, 67, 56, 53, 52, 48, 52, 55, 56, 56, 55, 55, 56, 54, 49,
+        101, 100, 69, 52, 65, 70, 102, 65, 102, 48, 52, 97, 54, 56, 101, 97, 49, 57, 52, 66, 55,
+        65, 52, 48, 48, 52, 54, 52, 50, 52, 101, 217, 130, 48, 50, 99, 68, 70, 65, 51, 51, 51, 99,
+        49, 56, 56, 57, 51, 100, 57, 102, 51, 54, 48, 51, 53, 97, 51, 98, 55, 55, 48, 50, 51, 52,
+        56, 97, 67, 102, 70, 48, 70, 68, 53, 65, 50, 65, 69, 57, 99, 66, 67, 48, 69, 52, 56, 69,
+        53, 57, 100, 100, 48, 56, 53, 53, 56, 49, 97, 54, 48, 49, 53, 57, 66, 55, 102, 99, 67, 99,
+        53, 52, 68, 68, 48, 70, 65, 57, 52, 52, 51, 100, 50, 69, 51, 53, 55, 51, 51, 55, 56, 68,
+        54, 49, 69, 97, 49, 54, 101, 54, 53, 57, 68, 49, 54, 100, 48, 48, 48, 57, 65, 52, 48, 66,
+        55, 55, 53, 48, 66, 67, 67, 69, 65, 69,
+    ];
+
     // Note: MessagePack messages can be visualized using the message pack visualizer at
     // https://sugendran.github.io/msgpack-visualizer/. Rust arrays can be copy&pasted and converted
     // to base64 using the following one-liner: `import base64; base64.b64encode(bytes([129, 0,
@@ -305,12 +427,12 @@ mod tests {
 
     #[test]
     fn v1_0_0_can_decode_current_handshake() {
+        let mut rng = crate::new_rng();
         let modern_handshake = Message::<protocol::Message>::Handshake {
             network_name: "example-handshake".to_string(),
             public_addr: ([12, 34, 56, 78], 12346).into(),
             protocol_version: ProtocolVersion::from_parts(5, 6, 7),
-            // TODO: Test _with_ handshake instead.
-            consensus_certificate: None,
+            consensus_certificate: Some(ConsensusCertificate::random(&mut rng)),
         };
 
         let legacy_handshake: V1_0_0_Message = roundtrip_message(&modern_handshake);
@@ -377,4 +499,69 @@ mod tests {
             }
         }
     }
+
+    #[test]
+    fn current_handshake_decodes_from_historic_v1_4_2() {
+        let modern_handshake: Message<protocol::Message> = deserialize_message(V1_4_2_HANDSHAKE);
+
+        match modern_handshake {
+            Message::Handshake {
+                network_name,
+                public_addr,
+                protocol_version,
+                consensus_certificate,
+            } => {
+                assert_eq!(network_name, "example-handshake");
+                assert_eq!(public_addr, ([12, 34, 56, 78], 12346).into());
+                assert_eq!(protocol_version, ProtocolVersion::from_parts(1, 4, 2));
+                let ConsensusCertificate {
+                    public_key,
+                    signature,
+                } = consensus_certificate.unwrap();
+
+                assert_eq!(
+                    public_key,
+                    PublicKey::from_hex(
+                        "020283c0d687933eb20a541c8540478877861ede4affaf04a68ea194b7a40046424e"
+                    )
+                    .unwrap()
+                );
+                assert_eq!(
+                    signature,
+                    Signature::from_hex(
+                        "02cdfa333c18893d9f36035a3b7702348acff0fd5a2ae9cbc0e48e59dd085581a6015\
+                        9b7fccc54dd0fa9443d2e3573378d61ea16e659d16d0009a40b7750bcceae"
+                    )
+                    .unwrap()
+                );
+            }
+            Message::Payload(_) => {
+                panic!("did not expect modern handshake to deserialize to payload")
+            }
+        }
+    }
+
+    fn roundtrip_certificate(use_human_readable: bool) {
+        let mut rng = crate::new_rng();
+        let certificate = ConsensusCertificate::random(&mut rng);
+
+        let deserialized = if use_human_readable {
+            let serialized = serde_json::to_string(&certificate).unwrap();
+            serde_json::from_str(&serialized).unwrap()
+        } else {
+            let serialized = bincode::serialize(&certificate).unwrap();
+            bincode::deserialize(&serialized).unwrap()
+        };
+        assert_eq!(certificate, deserialized);
+    }
+
+    #[test]
+    fn serde_json_roundtrip_certificate() {
+        roundtrip_certificate(true)
+    }
+
+    #[test]
+    fn bincode_roundtrip_certificate() {
+        roundtrip_certificate(false)
+    }
 }
