@@ -13,7 +13,6 @@ use std::{
 
 use datasize::DataSize;
 use derive_more::Into;
-use hex::FromHexError;
 use hex_fmt::HexList;
 use itertools::Itertools;
 use once_cell::sync::Lazy;
@@ -90,10 +89,10 @@ static ERA_END: Lazy<EraEnd> = Lazy::new(|| {
     EraEnd::new(era_report, next_era_validator_weights)
 });
 static FINALIZED_BLOCK: Lazy<FinalizedBlock> = Lazy::new(|| {
-    let deploy_hashes = vec![*Deploy::doc_example().id()];
+    let transfer_hashes = vec![*Deploy::doc_example().id()];
     let random_bit = true;
     let timestamp = *Timestamp::doc_example();
-    let block_payload = BlockPayload::new(deploy_hashes, vec![], vec![], random_bit);
+    let block_payload = BlockPayload::new(vec![], transfer_hashes, vec![], random_bit);
     let era_report = Some(EraReport::doc_example().clone());
     let era_id = EraId::from(1);
     let height = 10;
@@ -175,8 +174,8 @@ pub enum Error {
     DecodeFromJson(Box<dyn StdError>),
 }
 
-impl From<FromHexError> for Error {
-    fn from(error: FromHexError) -> Self {
+impl From<base16::DecodeError> for Error {
+    fn from(error: base16::DecodeError) -> Self {
         Error::DecodeFromJson(Box::new(error))
     }
 }
