@@ -5,12 +5,14 @@ use std::{
     collections::BTreeMap,
     hash::Hash,
     net::{IpAddr, Ipv4Addr, SocketAddr},
+    time::Duration,
 };
 
 use once_cell::sync::Lazy;
 use schemars::JsonSchema;
 use serde::{Deserialize, Serialize};
 
+use casper_hashing::Digest;
 use casper_types::{EraId, ProtocolVersion, PublicKey};
 
 use crate::{
@@ -18,7 +20,7 @@ use crate::{
         chainspec_loader::NextUpgrade,
         rpc_server::rpcs::docs::{DocExample, DOCS_EXAMPLE_PROTOCOL_VERSION},
     },
-    crypto::{hash::Digest, AsymmetricKeyExt},
+    crypto::AsymmetricKeyExt,
     types::{ActivationPoint, Block, BlockHash, NodeId, PeersMap, TimeDiff, Timestamp},
 };
 
@@ -46,6 +48,7 @@ static GET_STATUS_RESULT: Lazy<GetStatusResult> = Lazy::new(|| {
         our_public_signing_key: Some(PublicKey::doc_example().clone()),
         round_length: Some(TimeDiff::from(1 << 16)),
         version: crate::VERSION_STRING.as_str(),
+        node_uptime: Duration::from_secs(13),
     };
     GetStatusResult::new(status_feed, DOCS_EXAMPLE_PROTOCOL_VERSION)
 });
@@ -98,6 +101,8 @@ pub struct StatusFeed<I> {
     pub round_length: Option<TimeDiff>,
     /// The compiled node version.
     pub version: &'static str,
+    /// Time that passed since the node has started.
+    pub node_uptime: Duration,
 }
 
 impl<I> StatusFeed<I> {
@@ -106,6 +111,7 @@ impl<I> StatusFeed<I> {
         peers: BTreeMap<I, String>,
         chainspec_info: ChainspecInfo,
         consensus_status: Option<(PublicKey, Option<TimeDiff>)>,
+        node_uptime: Duration,
     ) -> Self {
         let (our_public_signing_key, round_length) = match consensus_status {
             Some((public_key, round_length)) => (Some(public_key), round_length),
@@ -118,6 +124,7 @@ impl<I> StatusFeed<I> {
             our_public_signing_key,
             round_length,
             version: crate::VERSION_STRING.as_str(),
+            node_uptime,
         }
     }
 }
@@ -170,6 +177,8 @@ pub struct GetStatusResult {
     pub next_upgrade: Option<NextUpgrade>,
     /// The compiled node version.
     pub build_version: String,
+    /// Time that passed since the node has started.
+    pub uptime: TimeDiff,
 }
 
 impl GetStatusResult {
@@ -183,7 +192,13 @@ impl GetStatusResult {
             our_public_signing_key: status_feed.our_public_signing_key,
             round_length: status_feed.round_length,
             next_upgrade: status_feed.chainspec_info.next_upgrade,
+            uptime: status_feed.node_uptime.into(),
+            #[cfg(not(test))]
             build_version: crate::VERSION_STRING.clone(),
+
+            //  Prevent these values from changing between test sessions
+            #[cfg(test)]
+            build_version: String::from("1.0.0-xxxxxxxxx@DEBUG"),
         }
     }
 }
